#include "Dark.h"

namespace Dark
{
  Player::Player() : Entity(),
		     FRICTION(400),
		     MAXSPEED(60.0f),
		     ACCELERATION(800)
  {
    sprite = new SpriteAnimation("player.png", FILTER_NONE, 64, 64);
    sprite->Add("idle", 0, 0, 0.35f);
    sprite->Play("idle");
    SetGraphic(sprite);

    AddTag("player");

    footsteps = Audio::NewDeck(Assets::RequestAudio("footsteps.ogg"));
    footsteps->SetLoops(0); //loop indefinitely
    
    SetCollider(new RectangleCollider(32, 32));
  }

  void Player::Update()
  {
    Entity::Update();

    if (Input::IsKeyMaskHeld("left"))
      {
	velocity.x -= ACCELERATION * Monocle::deltaTime;
      }
    if (Input::IsKeyMaskHeld("right"))
      {
	velocity.x += ACCELERATION * Monocle::deltaTime;
      }
    if (Input::IsKeyMaskHeld("up"))
      {
	velocity.y -= ACCELERATION * Monocle::deltaTime;
      }
    if (Input::IsKeyMaskHeld("down"))
      {
	velocity.y += ACCELERATION * Monocle::deltaTime;
      }
    
    // velocity will approach 0 in steps of FRICTION * dt
    velocity.x = APPROACH(velocity.x, 0, FRICTION * Monocle::deltaTime);
    velocity.y = APPROACH(velocity.y, 0, FRICTION * Monocle::deltaTime);

    bool xcol = false;
    bool ycol = false;

    position.x += velocity.x * Monocle::deltaTime;
    while (Collide("Solid"))
      {
	xcol = true;
	if (velocity.x == 0) { break; }
	//printf("collision1\n");
	position.x -= SIGN(velocity.x, 0.1);
      }
    if (xcol) {velocity.x = 0;}

    position.y += velocity.y * Monocle::deltaTime;
    while (Collide("Solid"))
      {
	ycol = true;
	if (velocity.y == 0) { break; }
	//printf("collision2\n");
	position.y -= SIGN(velocity.y, 0.1);
      }
    if (ycol) {velocity.y = 0;}

    if (velocity.GetSquaredMagnitude() > 20)
      footsteps->Play();
    else
      footsteps->Pause(); //Stop()?

    //Scene::GetCamera()->position = position;
  }

  Creature::Creature() : Entity(),
			 FRICTION(800),
			 MAXSPEED(30.0f),
			 ACCELERATION(1200)			 
  {
    sprite = new SpriteAnimation("creature.png", FILTER_NONE, 64, 64);
    sprite->Add("idle", 0, 0, 0.35f);
    sprite->Play("idle");
    SetGraphic(sprite);

    AddTag("creature");
    SetCollider(new RectangleCollider(32, 32));

    skitter1 = Audio::NewDeck(Assets::RequestAudio("skitter1.ogg"));
    skitter1->SetLoops(0); //loop indefinitely

    velocity = Vector2::Random() * ACCELERATION;

    //set aiTime to random so creatures tick at different times
    aiTime= (float(rand()) / float(RAND_MAX)) * 1.0f;
  }

  void Creature::Update()
  {
    Entity::Update();

    aiTime += Monocle::deltaTime;
    if (aiTime > 1.0f)
      {
	switch (rand() % 2)
	  {
	  case 0: // idle
	    //printf("idle\n");
	    direction = Vector2::zero;
	    state = "idle";
	    break;
	  case 1: // move
	    if (state != "wander") {
	      //printf("wander\n");
	      direction = Vector2::Random();
	      state = "wander";
	      break;
	    }
	  }
	
	aiTime = 0.0f;
      }
    
    velocity += direction * ACCELERATION * Monocle::deltaTime;
    
    velocity.x = APPROACH(velocity.x, 0, FRICTION * Monocle::deltaTime);
    velocity.y = APPROACH(velocity.y, 0, FRICTION * Monocle::deltaTime);

    bool xcol = false;
    bool ycol = false;

    position.x += velocity.x * Monocle::deltaTime;
    while (Collide("Solid"))
      {
	xcol = true;
	if (velocity.x == 0) { break; }
	//printf("collision1\n");
	position.x -= SIGN(velocity.x, 0.1);
      }
    if (xcol) {velocity.x = 0;}

    position.y += velocity.y * Monocle::deltaTime;
    while (Collide("Solid"))
      {
	ycol = true;
	if (velocity.y == 0) { break; }
	//printf("collision2\n");
	position.y -= SIGN(velocity.y, 0.1);
      }
    if (ycol) {velocity.y = 0;}

    if (velocity.GetSquaredMagnitude() > 20)
      skitter1->Play();
    else
      skitter1->Pause(); //Stop()?
  }

  // Scene

  DarkScene::DarkScene() : Scene()
  {
  }
  
  void DarkScene::Begin()
  {
    Scene::Begin();

    Graphics::Set2D(1024, 768);

    Text *inst2 = new Text("Press ESC to quit");
    inst2->position = Vector2(-492, -354); // 20px h, 30px v from U-L corner
    Add(inst2);

    Input::DefineMaskKey("left", KEY_LEFT);
    Input::DefineMaskKey("right", KEY_RIGHT);
    Input::DefineMaskKey("up", KEY_UP);
    Input::DefineMaskKey("down", KEY_DOWN);

    Input::DefineMaskKey("up", KEY_W);
    Input::DefineMaskKey("left", KEY_A);
    Input::DefineMaskKey("down", KEY_S);
    Input::DefineMaskKey("right", KEY_D);

    // level editor
    Add( levelEditor = new LevelEditor() );
    levelEditor->Disable();

    // load level from files
    Level::LoadProject("project.xml");
    Level::Load("level.xml", this);

    std::list<Entity*> *inv = GetAllTag("invisible");
    for (std::list<Entity*>::iterator i = inv->begin(); i != inv->end(); ++i)
      {
        Entity *e = (*i);
        Vector2 s = e->scale;
        e->SetCollider(new RectangleCollider(s.x * 64, s.y * 64));
      }
   
    Player *player = new Player;
    player->position = Graphics::GetScreenCenter();
    Add(player);

    Creature *creature = new Creature;
    creature->position = Graphics::GetScreenCenter();
    Add(creature);

    Creature *creature2 = new Creature;
    creature2->position = Graphics::GetScreenCenter();
    Add(creature2);

    Graphics::SetBackgroundColor(Color::green * 0.2f);

  }

  void DarkScene::Update()
  {
    Scene::Update();

    if (Input::IsKeyPressed(KEY_ESCAPE))
      Game::Quit();

    if (isPaused)
      {
	if (Input::IsKeyPressed(KEY_S) && Input::IsKeyHeld(KEY_LCTRL))
	  Level::Save();

	if (Input::IsKeyPressed(KEY_A))
	  Scene::GetCamera()->position.x -= 600;

	if (Input::IsKeyPressed(KEY_D))
	  Scene::GetCamera()->position.x += 600;
      }

    if (Input::IsKeyPressed(KEY_TAB))
      {
        // if we're not doing anything in the levelEditor...
        if (levelEditor->GetState() == FTES_NONE)
          {
            isPaused = !isPaused;
            
            if (isPaused)
	      levelEditor->Enable();
            else
	      levelEditor->Disable();
          }
      }

  }// DarkScene::Update

  Text::Text(const std::string& text, FontAsset* font)
    : Entity(), text(text)
  {
    if (font == NULL)
      this->font = Assets::RequestFont("LiberationSans-Regular.ttf", 18.0f);
    else
      this->font = font;
  }

  void Text::Render()
  {
    Graphics::PushMatrix();

    // place text directly relative to camera
    Graphics::Translate(scene->GetCamera()->position + position);

    Graphics::SetBlend(BLEND_ALPHA);
    Graphics::SetColor(Color::white);
    Graphics::BindFont(font);
    
    Graphics::RenderText(*font, text, 0, 0);
    Graphics::PopMatrix();
  }

}
